==> pdapi-0.1.3-sources.jar: ml/pluto7073/pdapi/DrinkUtil.java <==
package ml.pluto7073.pdapi;

import ml.pluto7073.pdapi.addition.DrinkAddition;
import ml.pluto7073.pdapi.addition.DrinkAdditions;
import ml.pluto7073.pdapi.entity.PDTrackedData;
import ml.pluto7073.pdapi.item.AbstractCustomizableDrinkItem;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2520;
import net.minecraft.class_2960;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Stack;

public final class DrinkUtil {

    private static final HashMap<String, Converter> OLD_CONVERSION_REGISTRY = new HashMap<>();
    private static final double CAFFEINE_HALF_LIFE_TICKS = 2500.0;

    public static void convertStackFromPlutosCoffee(class_1799 stack) {
        class_2487 nbt = stack.method_7948();
        if (!nbt.method_10545("Coffee")) return;
        Stack<String> currentPath = new Stack<>();
        currentPath.push("Coffee");
        class_2487 oldCoffeeData = nbt.method_10562("Coffee");
        handleCompound(currentPath, oldCoffeeData);
        nbt.method_10566(AbstractCustomizableDrinkItem.DRINK_DATA_NBT_KEY, oldCoffeeData);
        nbt.method_10551("Coffee");
    }

    private static void handleCompound(Stack<String> currentPath, class_2487 compound) {
        for (String key : compound.method_10541()) {
            class_2520 element = compound.method_10580(key);
            if (element == null) continue;
            currentPath.push(key);
            String current = convertPathStackToString(currentPath);
            if (OLD_CONVERSION_REGISTRY.containsKey(current)) {
                element = OLD_CONVERSION_REGISTRY.get(current).convert(element);
                compound.method_10566(key, element);
                continue;
            }
            if (element.method_10711() == class_2487.field_33260) {
                handleCompound(currentPath, (class_2487) element);
            }
            currentPath.pop();
        }
    }

    public static DrinkAddition[] getAdditionsFromStack(class_1799 stack) {
        convertStackFromPlutosCoffee(stack);
        class_2487 drinkData = stack.method_7911(AbstractCustomizableDrinkItem.DRINK_DATA_NBT_KEY);
        class_2499 additions = drinkData.method_10554(DrinkAdditions.ADDITIONS_NBT_KEY, class_2520.field_33258);
        ArrayList<DrinkAddition> additionsList = new ArrayList<>();
        for (int i = 0; i < additions.size(); i++) {
            String id = additions.method_10608(i);
            class_2960 identifier = new class_2960(id);
            additionsList.add(DrinkAdditions.get(identifier));
        }
        return additionsList.toArray(new DrinkAddition[0]);
    }

    public static void registerOldToNewConverter(String nbtPath, Converter converter) {
        OLD_CONVERSION_REGISTRY.put(nbtPath, converter);
    }

    private static String convertPathStackToString(Stack<String> stack) {
        if (stack.isEmpty()) return "";
        StringBuilder builder = new StringBuilder(stack.get(0));
        for (int i = 1; i < stack.size(); i++) {
            builder.append("/").append(stack.get(i));
        }
        return builder.toString();
    }

    public static class_2520 stringAsNbt(String s) {
        class_2487 compound = new class_2487();
        compound.method_10582("string", s);
        return compound.method_10580("string");
    }

    public static void setPlayerCaffeine(class_1657 player, float caffeine) {
        player.method_5841().method_12778(PDTrackedData.PLAYER_TICKS_SINCE_LAST_CAFFEINE, 0);
        player.method_5841().method_12778(PDTrackedData.PLAYER_CAFFEINE_AMOUNT, caffeine);
        player.method_5841().method_12778(PDTrackedData.PLAYER_ORIGINAL_CAFFEINE_AMOUNT, caffeine);
    }

    public static float calculateCaffeineDecay(int ticks, float originalCaffeine) {
        double exp = Math.pow(0.5, ticks / CAFFEINE_HALF_LIFE_TICKS);
        return (float) (exp * originalCaffeine);
    }

    public static float getPlayerCaffeine(class_1657 player) {
        return player.method_5841().method_12789(PDTrackedData.PLAYER_CAFFEINE_AMOUNT);
    }

    public interface Converter<T extends class_2520> {
        T convert(T startingElement);
    }

}

==> pdapi-1.20.1+0.1.6-sources.jar: ml/pluto7073/pdapi/DrinkUtil.java <==
package ml.pluto7073.pdapi;

import com.google.gson.JsonObject;
import ml.pluto7073.pdapi.addition.DrinkAddition;
import ml.pluto7073.pdapi.addition.DrinkAdditions;
import ml.pluto7073.pdapi.entity.PDTrackedData;
import ml.pluto7073.pdapi.item.AbstractCustomizableDrinkItem;
import ml.pluto7073.pdapi.item.PDItems;
import ml.pluto7073.pdapi.specialty.SpecialtyDrink;
import net.minecraft.class_1263;
import net.minecraft.class_1277;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2519;
import net.minecraft.class_2520;
import net.minecraft.class_2960;
import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Function;

public final class DrinkUtil {

    private static final HashMap<String, Converter> OLD_CONVERSION_REGISTRY = new HashMap<>();
    private static final double CAFFEINE_HALF_LIFE_TICKS = 2500.0;

    public static void convertStackFromPlutosCoffee(class_1799 stack) {
        class_2487 nbt = stack.method_7948();
        if (!nbt.method_10545("Coffee")) return;
        Stack<String> currentPath = new Stack<>();
        currentPath.push("Coffee");
        class_2487 oldCoffeeData = nbt.method_10562("Coffee");
        handleCompound(currentPath, oldCoffeeData);
        nbt.method_10566(AbstractCustomizableDrinkItem.DRINK_DATA_NBT_KEY, oldCoffeeData);
        nbt.method_10551("Coffee");
    }

    private static void handleCompound(Stack<String> currentPath, class_2487 compound) {
        for (String key : compound.method_10541()) {
            class_2520 element = compound.method_10580(key);
            if (element == null) continue;
            currentPath.push(key);
            String current = convertPathStackToString(currentPath);
            if (OLD_CONVERSION_REGISTRY.containsKey(current)) {
                element = OLD_CONVERSION_REGISTRY.get(current).convert(element);
                compound.method_10566(key, element);
                continue;
            }
            if (element.method_10711() == class_2487.field_33260) {
                handleCompound(currentPath, (class_2487) element);
            }
            currentPath.pop();
        }
    }

    public static class_1263 copyContainerContents(class_1263 source) {
        class_1263 container = new class_1277(source.method_5439());
        for (int i = 0; i < source.method_5439(); i++) {
            container.method_5447(i, source.method_5438(i).method_7972());
        }
        return container;
    }

    public static DrinkAddition[] getAdditionsFromStack(class_1799 stack) {
        convertStackFromPlutosCoffee(stack);
        class_2487 drinkData = stack.method_7911(AbstractCustomizableDrinkItem.DRINK_DATA_NBT_KEY);
        class_2499 additions = drinkData.method_10554(DrinkAdditions.ADDITIONS_NBT_KEY, class_2520.field_33258);
        ArrayList<DrinkAddition> additionsList = new ArrayList<>();
        for (int i = 0; i < additions.size(); i++) {
            String id = additions.method_10608(i);
            class_2960 identifier = new class_2960(id);
            additionsList.add(DrinkAdditions.get(identifier));
        }
        return additionsList.toArray(new DrinkAddition[0]);
    }

    public static void registerOldToNewConverter(String nbtPath, Converter converter) {
        OLD_CONVERSION_REGISTRY.put(nbtPath, converter);
    }

    private static String convertPathStackToString(Stack<String> stack) {
        if (stack.isEmpty()) return "";
        StringBuilder builder = new StringBuilder(stack.get(0));
        for (int i = 1; i < stack.size(); i++) {
            builder.append("/").append(stack.get(i));
        }
        return builder.toString();
    }

    public static class_2520 stringAsNbt(String s) {
        class_2487 compound = new class_2487();
        compound.method_10582("string", s);
        return compound.method_10580("string");
    }

    public static void setPlayerCaffeine(class_1657 player, float caffeine) {
        player.method_5841().method_12778(PDTrackedData.PLAYER_TICKS_SINCE_LAST_CAFFEINE, 0);
        player.method_5841().method_12778(PDTrackedData.PLAYER_CAFFEINE_AMOUNT, caffeine);
        player.method_5841().method_12778(PDTrackedData.PLAYER_ORIGINAL_CAFFEINE_AMOUNT, caffeine);
    }

    public static float calculateCaffeineDecay(int ticks, float originalCaffeine) {
        double exp = Math.pow(0.5, ticks / CAFFEINE_HALF_LIFE_TICKS);
        return (float) (exp * originalCaffeine);
    }

    public static float getPlayerCaffeine(class_1657 player) {
        return player.method_5841().method_12789(PDTrackedData.PLAYER_CAFFEINE_AMOUNT);
    }

    public static SpecialtyDrink getSpecialDrink(class_1799 stack) {
        class_2487 nbt = stack.method_7948();
        String id = nbt.method_10558("Drink");
        SpecialtyDrink drink = SpecialtyDrink.DRINKS.get(new class_2960(id));
        if (drink == null) throw new IllegalArgumentException("Drink " + id + " does not exist");
        return drink;
    }

    public static class_1799 setSpecialDrink(class_1799 stack, SpecialtyDrink drink) {
        class_2487 nbt = stack.method_7948();
        nbt.method_10566("Drink", class_2519.method_23256(drink.id().toString()));
        stack.method_7980(nbt);
        return stack;
    }

    public static int getDrinkColor(class_1799 stack) {
        if (!stack.method_31574(PDItems.SPECIALTY_DRINK)) return -1;
        try {
            SpecialtyDrink drink = getSpecialDrink(stack);
            return drink.color();
        } catch (IllegalArgumentException e) {
            return 0xf918c5;
        }
    }

    public interface Converter<T extends class_2520> {
        T convert(T startingElement);
    }

}
